library(plotrix)  #package to draw circles easily
library(doParallel)  # Package to parallelize computations
library(TDA)  #package to compute  Delaunay-complex filtrations
library(R.matlab)  # To import .mat file
library(fda)  # Package for functional data analysis
library(depthTools)  # Additional options to functional data analysis


# Modification of the TDA function 'alphaComplexDiag' to be compatible with ppp objects from the R-package spatstat
# and to return a matrix in place of object of class diagram (easier to handle) This function take a ppp object and
# return the corresponding persistence diagram (as in Section 1.1.1 of the article)
alphaDiag <- function(PP) {
    if (is.ppp(PP) == TRUE) {
        sim_PP_coord <- cbind(PP$x, PP$y)
    } else {
        sim_PP_coord <- cbind(PP[, 1], PP[, 2])
    }
    Diag <- cbind(alphaComplexDiag(sim_PP_coord)$diagram[, 1], sqrt(alphaComplexDiag(sim_PP_coord)$diagram[, 2]), 
                  sqrt(alphaComplexDiag(sim_PP_coord)$diagram[, 3]))
    colnames(Diag) <- c("dimension", "Birth", "Death")
    return(Diag)
}

# Definition of the Accumulated Persistent Function (APF).  m: the arguments of the APF, Diag: the persistence
# diagramm, k: the dimension of the topological features
APFtemp <- function(m, Diag, k) {
    Diagk <- Diag[Diag[, 1] == k, ]
    if (is.matrix(Diagk) == TRUE) {
        lifetime <- Diagk[, 3] - Diagk[, 2]
        meanage <- 0.5 * (Diagk[, 3] + Diagk[, 2])
        sum(lifetime[meanage <= m])
    } else {
        lifetime <- Diagk[3] - Diagk[2]
        meanage <- 0.5 * (Diagk[3] + Diagk[2])
        sum(lifetime[meanage <= m])
    }
}
APF <- Vectorize(APFtemp, vectorize.args = "m", "Diag")

# Loading of the persistence diagrams
CaseInformation <- readMat("CaseInformation.mat")
H0.PD.temp <- readMat("H0PersistenceDiagrams.mat")
H1.PD.temp <- readMat("H1PersistenceDiagrams.mat")

# I convert into a list of matrices
H0.PD.full <- NULL
H1.PD.full <- NULL
for (i in (1:length(H0.PD.temp$H0Array))) {
    H0.PD.full[[i]] <- cbind(rep(0, length(H0.PD.temp$H0Array[[i]][[1]][, 1])), H0.PD.temp$H0Array[[i]][[1]])
}
for (i in (1:length(H1.PD.temp$H1Array))) {
    H1.PD.full[[i]] <- cbind(rep(1, length(H1.PD.temp$H1Array[[i]][[1]][, 1])), H1.PD.temp$H1Array[[i]][[1]])
}

PD.brain.full <- NULL
for (i in 1:length(H0.PD.full)) {
    PD.brain.full[[i]] <- rbind(rbind(H0.PD.full[[i]], H1.PD.full[[i]]))
}

# #### Empty subject detection, I take off them from the sample
Emptysubject0 <- NULL
Emptysubject1 <- NULL
for (i in 1:109) {
    Emptysubject0[i] <- length(H0.PD.full[[i]])
}
Emptysubject0 <- which(Emptysubject0 == 0)
Emptysubject1 <- which(Emptysubject1 == 0)
H0.PD <- H0.PD.full[-Emptysubject0]
H1.PD <- H1.PD.full[-Emptysubject0]

# I replace the value -1 in the persistence diagram by Inf, these values correspond to unobserved death time.
for (i in (1:length(H1.PD))) {
    H1.PD[[i]][which(H1.PD[[i]][, 3] == -1), 3] <- Inf
}

#### I compute all the persistence diagrams that are non empty
PD.brain <- NULL
for (i in 1:length(H0.PD)) {
    PD.brain[[i]] <- rbind(rbind(H0.PD[[i]], H1.PD[[i]]))
}

# Index of the female and male subjects
female <- which(CaseInformation$Sexes[-Emptysubject0] == 1)
male <- which(CaseInformation$Sexes[-Emptysubject0] == 2)

# Computation of APF0 corresponding to female and male subject
mm0 <- seq(0, 160, 0.005)
registerDoParallel(cores = 3)
APF0.brain.female <- foreach(i = female, .combine = rbind) %dopar% {
    APF(mm0, PD.brain[[i]], 0)
}
APF0.brain.male <- foreach(i = male, .combine = rbind) %dopar% {
    APF(mm0, PD.brain[[i]], 0)
}

# Computation of APF0 corresponding to female and male subject
mm1 <- seq(0, 27, 1e-04)
registerDoParallel(cores = 3)
APF1.brain.female <- foreach(i = female, .combine = rbind) %dopar% {
    APF(mm1, PD.brain[[i]], 1)
}
APF1.brain.male <- foreach(i = male, .combine = rbind) %dopar% {
    APF(mm1, PD.brain[[i]], 1)
}


# Code to extract the topological features with the 100th largest lifetimes as in setting A
thres.PD <- function(Diag, kk, thres) {
    Diagk <- Diag[Diag[, 1] == kk, ]
    PD.thres <- NULL
    index <- sort(Diagk[, 3] - Diagk[, 2], decreasing = FALSE, index.return = T)$ix
    PD.thres <- Diagk[index[(length(index) - (thres - 1)):length(index)], ]
    return(PD.thres)
}
H0.brain.female.dominant <- foreach(i = 1:length(H0.PD[female])) %do% {
    Diag <- H0.PD[female]
    thres.PD(Diag[[i]], kk = 0, thres = 100)
}
H0.brain.male.dominant <- foreach(i = 1:length(H0.PD[male])) %do% {
    Diag <- H0.PD[male]
    thres.PD(Diag[[i]], kk = 0, thres = 100)
}
H1.brain.female.dominant <- foreach(i = 1:length(H1.PD[female])) %do% {
    Diag <- H1.PD[female]
    thres.PD(Diag[[i]], kk = 1, thres = 100)
}
H1.brain.male.dominant <- foreach(i = 1:length(H1.PD[male])) %do% {
    Diag <- H1.PD[male]
    thres.PD(Diag[[i]], kk = 1, thres = 100)
}


# Code to detect the outliers in setting C

# Modification of the fbplot function from the R-package fda in order to have more options on the plot (color, line
# width ... etc) The first three functions are used in the last one, fbplotperso
combinatperso <- function(n, p) {
    if (n < p) {
        combinat <- 0
    } else {
        combinat <- exp(lfactorial(n) - (lfactorial(p) + lfactorial(n - p)))
    }
}
fBD2perso <- function(data) {
    p <- dim(data)[1]
    n <- dim(data)[2]
    rmat <- apply(data, 1, rank)
    down <- apply(rmat, 1, min) - 1
    up <- n - apply(rmat, 1, max)
    (up * down + n - 1)/combinatperso(n, 2)
    
}
fMBDperso <- function(data) {
    p <- dim(data)[1]
    n <- dim(data)[2]
    rmat <- apply(data, 1, rank)
    down <- rmat - 1
    up <- n - rmat
    (rowSums(up * down)/p + n - 1)/combinatperso(n, 2)
}
fbplotperso <- function(fit, x = NULL, method = "MBD", depth = NULL, plot = TRUE, lwdenv = 2, colbordenv = 4, colenv = 6, 
    colbornes = 4, ltybornes = 1, lwdbornes = 2, prob = 0.5, lwdmeancurve = 2, outlierlwd = 4, outliercol = 2, fullout = TRUE, 
    fact = 1.5, xlim = c(1, nrow(fit)), ylim = c(min(fit) - 0.5 * diff(range(fit)), max(fit) + 0.5 * diff(range(fit))), 
    ...) {
    tp <- dim(fit)[1]
    n <- dim(fit)[2]
    if (length(x) == 0) {
        x <- 1:tp
    }
    # if (length(depth) != length(fit[1,])) { return(' error : depth must be calculated correctly ') }
    if (length(depth) == 0) {
        if (method == "BD2") {
            depth <- fBD2perso(fit)
        } else if (method == "MBD") {
            depth <- fMBDperso(fit)
        } else if (method == "Both") {
            depth <- round(fBD2perso(fit), 4) * 10000 + fMBDperso(fit)
        }
    }
    dp_s <- sort(depth, decreasing = TRUE)
    index <- order(depth, decreasing = TRUE)
    med <- depth == max(depth)
    medavg <- matrix(fit[, med], ncol = sum(med), nrow = tp)
    y <- apply(medavg, 1, mean)
    if (plot) {
        plot(x, y, lty = 1, lwd = 2, col = 1, type = "l", xlim, ylim, ...)
    }
    xx <- NULL
    yy <- NULL
    for (pp in 1:length(prob)) {
        m <- ceiling(n * prob[pp])
        center <- fit[, index[1:m]]
        out <- fit[, index[(m + 1):n]]
        inff <- apply(center, 1, min)
        supp <- apply(center, 1, max)
        
        
        # Outliers detection
        if (prob[pp] == 0.5) {
            dist <- fact * (supp - inff)
            upper <- supp + dist
            lower <- inff - dist
            outly <- (fit < lower) + (fit > upper)
            outcol <- colSums(outly)
            remove <- (outcol > 0)
            colum <- 1:n
            outpoint <- colum[remove == 1]
            out <- fit[, remove]
            woout <- fit
            good <- woout[, (remove == 0), drop = FALSE]
            maxcurve <- apply(good, 1, max)
            mincurve <- apply(good, 1, min)
            barval <- (x[1] + x[tp])/2
            bar <- which(sort(c(x, barval)) == barval)[1]
            if (plot) {
                # code not used but present in the function fbplot of the R-package fda lines(c(x[bar], x[bar]), c(maxcurve[bar],
                # supp[bar]), col = barcol, lwd = 2) lines(c(x[bar], x[bar]), c(mincurve[bar], inff[bar]), col = barcol, lwd = 2)
            }
        }
        # Outliers detection end
        
        # Plot of the central region
        xx[[pp]] <- c(x, x[order(x, decreasing = TRUE)])
        suppinv <- supp[order(x, decreasing = TRUE)]
        yy[[pp]] <- c(inff, suppinv)
        if (plot) {
            if (prob[pp] == 0.5) {
                polygon(xx[[pp]], yy[[pp]], col = colenv, border = colbordenv, lwd = lwdenv)
            } else {
                polygon(xx[[pp]], yy[[pp]], col = colenv, border = NA)
            }
        }
        # Plot of the central region - end
        
    }
    if (plot) {
        lines(x, fit[, index[1]], lty = 1, lwd = lwdmeancurve, col = 1, type = "l")
        lines(x, maxcurve, col = colbornes, lwd = lwdbornes, lty = ltybornes)
        lines(x, mincurve, col = colbornes, lwd = lwdbornes, lty = ltybornes)
        if (fullout) {
            if (sum(outly) > 0) {
                if (plot) {
                  matlines(x, out, lty = 2, col = outliercol, lwd = outlierlwd, type = "l")
                }
            }
        }
    }
    return(list(outpoint = outpoint, medcurve = which(med), envXX = xx, envYY = yy, maxcurve = maxcurve, mincurve = mincurve))
}



MBD0female <- MBD(APF0.brain.female, plotting = F)$MBD
FB.0female <- fbplotperso(t(APF0.brain.female)[seq(1, length(mm0), by = 8), ], x = mm0[seq(1, length(mm0), by = 8)], 
    depth = MBD0female, plot = FALSE)
FB.0female$outpoint  # Display index of the curves detected as outliers

MBD0male <- MBD(APF0.brain.male, plotting = F)$MBD
FB.0male <- fbplotperso(t(APF0.brain.male)[seq(1, length(mm0), by = 8), ], x = mm0[seq(1, length(mm0), by = 8)], depth = MBD0male, 
    plot = FALSE)
FB.0male$outpoint  # Display index of the curves detected as outliers

MBD1female <- MBD(APF1.brain.female, plotting = F)$MBD
FB.1female <- fbplotperso(t(APF1.brain.female)[seq(1, length(mm1), by = 100), ], x = mm1[seq(1, length(mm1), by = 100)], 
    depth = MBD1female, plot = FALSE)
FB.1female$outpoint  # Display index of the curves detected as outliers

MBD1male <- MBD(APF1.brain.male, plotting = F)$MBD
FB.1male <- fbplotperso(t(APF1.brain.male)[seq(1, length(mm1), by = 100), ], x = mm1[seq(1, length(mm1), by = 100)], 
    depth = MBD1male, plot = FALSE)
FB.1male$outpoint  # Display index of the curves detected as outliers


######################### Two sample test

# Function performing the two-sample test based on the function KS defined as in (8). Return an estimated p-value 
# mm: arguments of the APF, Diag: list containing the repetition of D0, Qiag: list containing the repetition of E0 B:
# number of iteration of the bootstrap method
KS.permtest.brain <- function(mm, Diag, Qiag, kk, B, mcores) {
    LL1 <- length(Diag)
    LL2 <- length(Qiag)
    Diag_APF <- foreach(ii = 1:LL1, .combine = rbind) %do% {
        APF(mm, Diag = Diag[[ii]], k = kk)
    }
    Qiag_APF <- foreach(ii = 1:LL2, .combine = rbind) %do% {
        APF(mm, Diag = Qiag[[ii]], k = kk)
    }
    Mat_APF <- rbind(Diag_APF, Qiag_APF)
    KS0 <- sqrt(LL1 * LL2/(LL1 + LL2)) * max(abs(colMeans(Mat_APF[1:LL1, ]) - colMeans(Mat_APF[(LL1 + 1):(LL1 + LL2), 
        ])))
    Bootapf <- function(MMat_APF) {
        bootMat_APF <- MMat_APF[sample(nrow(MMat_APF), nrow(MMat_APF), replace = FALSE, prob = NULL), ]
        sqrt(LL1 * LL2/(LL1 + LL2)) * max(abs(colMeans(bootMat_APF[1:LL1, ]) - colMeans(bootMat_APF[(LL1 + 1):(LL1 + 
            LL2), ])))
    }
    registerDoParallel(cores = mcores)
    KS <- foreach(ii = 1:B, .combine = cbind) %dopar% {
        Bootapf(Mat_APF)
    }
    return(mean(KS > KS0))
}

# Setting A
KS.permtest.brain(seq(0, 137, 0.001), H0.brain.female.dominant, H0.brain.male.dominant, 0, 10000, mcores = 1)
KS.permtest.brain(seq(0, 60, 0.001), H0.brain.female.dominant, H0.brain.male.dominant, 0, 10000, mcores = 2)
KS.permtest.brain(seq(0, 25, 0.001), H1.brain.female.dominant, H1.brain.male.dominant, 1, 10000, mcores = 2)
KS.permtest.brain(seq(15, 25, 0.001), H1.brain.female.dominant, H1.brain.male.dominant, 1, 10000, mcores = 2)


# Setting B
KS.permtest.brain(seq(0, 137, 0.001), PD.brain[female], PD.brain[male], 0, 10000, mcores = 2)
KS.permtest.brain(seq(0, 60, 0.001), PD.brain[female], PD.brain[male], 0, 10000, mcores = 2)
KS.permtest.brain(seq(0, 25, 0.001), PD.brain[female], PD.brain[male], 1, 10000, mcores = 2)
KS.permtest.brain(seq(15, 25, 0.001), PD.brain[female], PD.brain[male], 1, 10000, mcores = 2)


# Setting C
KS.permtest.brain(seq(0, 137, 0.001), PD.brain[female[-FB.0female$outpoint]], PD.brain[male[-FB.0male$outpoint]], 0, 
    10000, mcores = 2)
KS.permtest.brain(seq(0, 60, 0.001), PD.brain[female[-FB.0female$outpoint]], PD.brain[male[-FB.0male$outpoint]], 0, 10000, 
    mcores = 2)
KS.permtest.brain(seq(0, 25, 0.001), PD.brain[female[-FB.1female$outpoint]], PD.brain[male[-FB.1male$outpoint]], 1, 10000, 
    mcores = 2)
KS.permtest.brain(seq(15, 25, 0.001), PD.brain[female[-FB.1female$outpoint]], PD.brain[male[-FB.1male$outpoint]], 1, 
    10000, mcores = 2)















