library(plotrix)  #package to draw circles easily
library(doParallel)  # Package to parallelize computations
library(TDA)  #package to compute  Delaunay-complex filtrations


# Modification of the TDA function 'alphaComplexDiag' to be compatible with ppp objects from the R-package spatstat
# and to return a matrix in place of object of class diagram (easier to handle) This function take a ppp object and
# return the corresponding persistence diagram (as in Section 1.1.1 of the article)
alphaDiag <- function(PP) {
    if (is.ppp(PP) == TRUE) {
        sim_PP_coord <- cbind(PP$x, PP$y)
    } else {
        sim_PP_coord <- cbind(PP[, 1], PP[, 2])
    }
    Diag <- cbind(alphaComplexDiag(sim_PP_coord)$diagram[, 1], sqrt(alphaComplexDiag(sim_PP_coord)$diagram[, 2]), 
                  sqrt(alphaComplexDiag(sim_PP_coord)$diagram[, 3]))
    colnames(Diag) <- c("dimension", "Birth", "Death")
    return(Diag)
}

# Definition of the Accumulated Persistent Function (APF).  m: the arguments of the APF, Diag: the persistence
# diagram, k: the dimension of the topological features
APFtemp <- function(m, Diag, k) {
    Diagk <- Diag[Diag[, 1] == k, ]
    if (is.matrix(Diagk) == TRUE) {
        lifetime <- Diagk[, 3] - Diagk[, 2]
        meanage <- 0.5 * (Diagk[, 3] + Diagk[, 2])
        sum(lifetime[meanage <= m])
    } else {
        lifetime <- Diagk[3] - Diagk[2]
        meanage <- 0.5 * (Diagk[3] + Diagk[2])
        sum(lifetime[meanage <= m])
    }
}
APF <- Vectorize(APFtemp, vectorize.args = "m", "Diag")

# Function to directly return a matrix with the function gridDiag from the R-package TDA
gridDiagram <- function(XX, lim, by) {
    gridDiag(XX, distFct, lim = lim, by = by, sublevel = TRUE, diagLimit = Inf)$diagram
}
# function to plot points on the union of two circles
two.unif.circle <- function(nbrepoint1, nbrepoint2, r1, r2, c1, c2) {
    if (length(c1) == 1) {
        circle1 <- c1 + circleUnif(nbrepoint1, r = r1)
    } else {
        circle1 <- cbind(rep(c1[1], nbrepoint1), rep(c1[2], nbrepoint1)) + circleUnif(nbrepoint1, r = r1)
    }
    if (length(c2) == 1) {
        circle2 <- c2 + circleUnif(nbrepoint2, r = r2)
    } else {
        circle2 <- cbind(rep(c2[1], nbrepoint2), rep(c2[2], nbrepoint2)) + circleUnif(nbrepoint2, r = r2)
    }
    return(rbind(circle1, circle2))
}

# input data
n <- 300  # or 500
repartition <- sample(c(1, 2), n, replace = T)
XX <- two.unif.circle(nbrepoint1 = length(repartition[repartition == 1]), nbrepoint2 = length(repartition[repartition == 
    2]), r1 = 1, r2 = 0.8, c1 = c(-1.5, 0), c2 = c(1.5, 0))

### Plot of the set C
par(mar = c(0, 0, 0, 0))
plot(XX, asp = 1, col = 0, pch = 16, ann = F, tcl = 0, col.axis = 0, axes = F)
draw.circle(-1.5, 0, 1, lwd = 6)
draw.circle(1.5, 0, 0.8, lwd = 6)

### Plot of the Point Pattern
par(mar = c(0, 0, 0, 0))
plot(XX, asp = 1, pch = 16, ann = F, tcl = 0, col.axis = 0, axes = F, cex = 1)

## Ranges of the grid
Xlim <- c(-2.6, 2.6)
Ylim <- c(-1.6, 1.6)
lim <- cbind(Xlim, Ylim)
by <- 0.03

# distance function
Diag <- gridDiag(XX, distFct, lim = lim, by = by, sublevel = TRUE, printProgress = TRUE)
Diag <- Diag$diagram
Diag <- Diag[Diag[, 1] == 1, ]  #Extract only the points corresponding to the holes


# confidence set
B <- 500
alpha <- 0.05
cc0 <- bootstrapDiagram(XX, distFct, lim = lim, by = by, sublevel = TRUE, parallel = TRUE, B = B, alpha = alpha, dimension = 1, 
    printProgress = TRUE)


# Function to plot the confidence region of the persistence diagram and on APF
conf.carre <- function(Diag, cc, col) {
    n <- length(Diag[, 1])
    for (i in 1:n) {
        xx <- Diag[i, 2]
        yy <- Diag[i, 3]
        polygon(c(xx - cc, xx + cc, xx + cc, xx - cc), c(yy - cc, yy - cc, yy + cc, yy + cc), border = col, lwd = 5)
    }
}
conf.losange <- function(Diag, cc, col) {
    n <- length(Diag[, 1])
    for (i in 1:n) {
        xx <- 0.5 * (Diag[i, 2] + Diag[i, 3])
        yy <- Diag[i, 3] - Diag[i, 2]
        ccx <- cc
        ccy <- 2 * cc
        polygon(c(xx - ccx, xx, xx + ccx, xx), c(yy, yy - ccy, yy, yy + ccy), border = col, lwd = 5)
    }
}

fupper <- function(m, Diag, cc) {
    ccx <- cc
    ccy <- 2 * cc
    meanage <- 0.5 * (Diag[, 2] + Diag[, 3])
    lifetime <- Diag[, 3] - Diag[, 2]
    meanagexx <- meanage - ccx
    lifetimeyy <- lifetime + ccy
    partialfunc <- NULL
    for (ii in 1:length(Diag[, 1])) {
        partialfunc <- rbind(partialfunc, ifelse(m <= meanagexx[ii], 0, ifelse(m <= meanage[ii], (ccy/ccx) * (m - meanage[ii]) + 
            lifetimeyy[ii], lifetimeyy[ii])))
    }
    colSums(partialfunc)
}

flower <- function(m, Diag, cc) {
    ccx <- cc
    ccy <- 2 * cc
    meanage <- 0.5 * (Diag[, 2] + Diag[, 3])
    lifetime <- Diag[, 3] - Diag[, 2]
    meanagexx <- meanage + ccx
    lifetimeyy <- pmax(lifetime - ccy, rep(0, length(lifetime)))
    partialfunc <- NULL
    for (ii in 1:length(Diag[, 1])) {
        partialfunc <- rbind(partialfunc, ifelse(m <= meanagexx[ii], 0, lifetimeyy[ii]))
    }
    colSums(partialfunc)
}

# Plot of the confidence region of the persistence diagram
par(mar = c(7, 7, 0.5, 0.5))
plot(Diag[, 2], Diag[, 3], xlim = c(-0.03, 1.1), ylim = c(0.05, 1.1), pch = 20, main = "", xlab = "", ylab = "", lwd = 4, 
    cex.axis = 2.5, mgp = c(3, 2, 0), asp = 1)
abline(a = 0, b = 1)
title(ylab = "Death time", mgp = c(4.5, 1, 0), cex.lab = 3)
title(xlab = "Birth time", mgp = c(5, 1, 0), cex.lab = 3)
conf.carre(Diag, cc0, col = "2")

# Plot of the confidence region of the rotated and rescaled persistence diagram
par(mar = c(7, 7, 0.5, 0.5))
plot(0.5 * (Diag[, 2] + Diag[, 3]), (Diag[, 3] - Diag[, 2]), xlim = c(-0.03, 0.7), ylim = c(-0.03, 1.2), pch = 20, asp = 1, 
    cex.lab = 2, cex = 2, main = "", xlab = "", ylab = "", lwd = 4, cex.axis = 2.5, mgp = c(3, 2, 0))
title(ylab = "Lifetime", mgp = c(4.5, 1, 0), cex.lab = 3)
title(xlab = "Meanage", mgp = c(5, 1, 0), cex.lab = 3)
conf.losange(Diag, cc0, col = "2")

# Plot of the confidence region for the APF
mm <- seq(-0.5, 2, 0.001)
plot(mm, APF(mm, Diag = Diag, k = 1), xlim = c(-0.03, 1.7), ylim = c(0.05, 5), type = "l", xlab = "", ylab = "", lwd = 0, 
    cex.axis = 2.5, mgp = c(3, 2, 0))
xx <- c(mm, rev(mm))
yy <- c(flower(mm, Diag, cc0), rev(fupper(mm, Diag, cc0)))
polygon(xx, yy, col = 16)
title(ylab = expression(APF[1](m)), mgp = c(4, 1, 0), cex.lab = 3)
title(xlab = expression("m"), mgp = c(5, 1, 0), cex.lab = 3)
lines(mm, fupper(mm, Diag, cc0), lwd = 6, col = 2)
lines(mm, flower(mm, Diag, cc0), lwd = 6, col = 2)
lines(mm, APF(mm, Diag = Diag, k = 1), col = 1, lwd = 8, lty = 2)
legend("topleft", c("Confidence region bounds", expression(APF[1])), lty = c(1, 2), col = c("2", "1"), y.intersp = 1.5, 
    lwd = 4, cex = 2.2, seg.len = 3)


